/*
 * Copyright (c) 2019 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.Controls 1.1
import QtQuick.Layouts 1.1
import Shotcut.Controls 1.0

KeyframableFilter {
    property string azimuth: '0'
    property string elevation: '1'
    property string width45: '2'
    property double azimuthDefault: 0.38
    property double elevationDefault: 0.33
    property double width45Default: 0.25

    keyframableParameters: [azimuth, elevation, width45]
    startValues: [0.5, 0.5, 0.5]
    middleValues: [azimuthDefault, elevationDefault, width45Default]
    endValues: [0.5, 0.5, 0.5]

    width: 350
    height: 100

    Component.onCompleted: {
        if (filter.isNew) {
            filter.set(azimuth, azimuthDefault)
            filter.set(elevation, elevationDefault)
            filter.set(width45, width45Default)
            filter.savePreset(preset.parameters)
        }
        setControls()
    }

    function setControls() {
        var position = getPosition()
        blockUpdate = true
        azimuthSlider.value = filter.getDouble(azimuth, position) * azimuthSlider.maximumValue
        elevationSlider.value = filter.getDouble(elevation, position) * elevationSlider.maximumValue
        width45Slider.value = filter.getDouble(width45, position) * width45Slider.maximumValue
        blockUpdate = false
        enableControls(isSimpleKeyframesActive())
    }

    function enableControls(enabled) {
        azimuthSlider.enabled = elevationSlider.enabled = width45Slider.enabled = enabled
    }

    function updateSimpleKeyframes() {
        updateFilter(azimuth, azimuthSlider.value / azimuthSlider.maximumValue, azimuthKeyframesButton)
        updateFilter(elevation, elevationSlider.value / elevationSlider.maximumValue, elevationKeyframesButton)
        updateFilter(width45, width45Slider.value / width45Slider.maximumValue, widthKeyframesButton)
    }

    GridLayout {
        anchors.fill: parent
        anchors.margins: 8
        columns: 4

        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }
        Preset {
            id: preset
            parameters: [azimuth, elevation, width45]
            Layout.columnSpan: 3
            onBeforePresetLoaded: {
                filter.resetProperty(azimuth)
                filter.resetProperty(elevation)
                filter.resetProperty(width45)
            }
            onPresetSelected: {
                setControls()
                initializeSimpleKeyframes()
            }
        }

        Label {
            text: qsTr('Light direction')
            Layout.alignment: Qt.AlignRight
        }
        SliderSpinner {
            id: azimuthSlider
            minimumValue: 0.00
            maximumValue: 1.00
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(azimuth, azimuthSlider.value / azimuthSlider.maximumValue, azimuthKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: azimuthSlider.value = azimuthDefault * azimuthSlider.maximumValue
        }
        KeyframesButton {
            id: azimuthKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(azimuth) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, azimuth, azimuthSlider.value / azimuthSlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Background lightness')
            Layout.alignment: Qt.AlignRight
        }
        SliderSpinner {
            id: elevationSlider
            minimumValue: 0.00
            maximumValue: 1.00
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(elevation, elevationSlider.value / elevationSlider.maximumValue, elevationKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: elevationSlider.value = elevationDefault * elevationSlider.maximumValue
        }
        KeyframesButton {
            id: elevationKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(elevation) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, elevation, elevationSlider.value / elevationSlider.maximumValue)
            }
        }

        Label {
            text: qsTr('Bump height')
            Layout.alignment: Qt.AlignRight
        }
        SliderSpinner {
            id: width45Slider
            minimumValue: 0.00
            maximumValue: 1.00
            stepSize: 0.01
            decimals: 2
            suffix: ' '
            onValueChanged: updateFilter(width45, width45Slider.value / width45Slider.maximumValue, widthKeyframesButton, getPosition())
        }
        UndoButton {
            onClicked: width45Slider.value = width45Default * width45Slider.maximumValue
        }
        KeyframesButton {
            id: widthKeyframesButton
            checked: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount(width45) > 0
            onToggled: {
                enableControls(true)
                toggleKeyframes(checked, width45, width45Slider.value / width45Slider.maximumValue)
            }
        }
        
        Item {
            Layout.fillHeight: true
        }
    }

    Connections {
        target: filter
        onInChanged: updateSimpleKeyframes()
        onOutChanged: updateSimpleKeyframes()
        onAnimateInChanged: updateSimpleKeyframes()
        onAnimateOutChanged: updateSimpleKeyframes()
    }

    Connections {
        target: producer
        onPositionChanged: setControls()
    }
}
